@@ -35,14 +35,11 @@
 #include "gusb-device-private.h"
 #include "gusb-util.h"
 
-static void g_usb_context_finalize (GObject *object);
-
-#define G_USB_CONTEXT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), G_USB_TYPE_CONTEXT, GUsbContextPrivate))
-
 enum {
 	PROP_0,
 	PROP_LIBUSB_CONTEXT,
-	PROP_DEBUG_LEVEL
+	PROP_DEBUG_LEVEL,
+	N_PROPERTIES
 };
 
 enum {
@@ -59,9 +56,10 @@ enum {
 struct _GUsbContextPrivate
 {
 	GPtrArray			*devices;
+	GHashTable			*dict_usb_ids;
 	GThread				*thread_event;
 	gboolean			 done_enumerate;
-	gboolean			 thread_event_run;
+	volatile gint			 thread_event_run;
 	guint				 hotplug_poll_id;
 	int				 debug_level;
 	libusb_context			*ctx;
@@ -69,17 +67,53 @@ struct _GUsbContextPrivate
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
+static GParamSpec *pspecs[N_PROPERTIES] = { NULL, };
 
-G_DEFINE_TYPE (GUsbContext, g_usb_context, G_TYPE_OBJECT)
+static void g_usb_context_initable_iface_init (GInitableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (GUsbContext, g_usb_context, G_TYPE_OBJECT,
+                         G_ADD_PRIVATE (GUsbContext)
+                         G_IMPLEMENT_INTERFACE(G_TYPE_INITABLE,
+                                               g_usb_context_initable_iface_init))
 
 /**
- * usb_context_get_property:
+ * g_usb_context_error_quark:
+ *
+ * Return value: Our personal error quark.
+ *
+ * Since: 0.1.0
  **/
+G_DEFINE_QUARK (g-usb-context-error-quark, g_usb_context_error)
+
 static void
-g_usb_context_get_property (GObject		*object,
-			    guint		 prop_id,
-			    GValue		*value,
-			    GParamSpec		*pspec)
+g_usb_context_dispose (GObject *object)
+{
+	GUsbContext *context = G_USB_CONTEXT (object);
+	GUsbContextPrivate *priv = context->priv;
+
+	/* this is safe to call even when priv->hotplug_id is unset */
+	if (g_atomic_int_dec_and_test (&priv->thread_event_run)) {
+		libusb_hotplug_deregister_callback (priv->ctx, priv->hotplug_id);
+		g_thread_join (priv->thread_event);
+	}
+
+	if (priv->hotplug_poll_id > 0) {
+		g_source_remove (priv->hotplug_poll_id);
+		priv->hotplug_poll_id = 0;
+	}
+
+	g_clear_pointer (&priv->devices, g_ptr_array_unref);
+	g_clear_pointer (&priv->dict_usb_ids, g_hash_table_unref);
+	g_clear_pointer (&priv->ctx, libusb_exit);
+
+	G_OBJECT_CLASS (g_usb_context_parent_class)->dispose (object);
+}
+
+static void
+g_usb_context_get_property (GObject    *object,
+                            guint       prop_id,
+                            GValue     *value,
+                            GParamSpec *pspec)
 {
 	GUsbContext *context = G_USB_CONTEXT (object);
 	GUsbContextPrivate *priv = context->priv;
@@ -97,22 +131,16 @@ g_usb_context_get_property (GObject		*object,
 	}
 }
 
-/**
- * usb_context_set_property:
- **/
 static void
-g_usb_context_set_property (GObject		*object,
-			   guint		 prop_id,
-			   const GValue		*value,
-			   GParamSpec		*pspec)
+g_usb_context_set_property (GObject      *object,
+                            guint         prop_id,
+                            const GValue *value,
+                            GParamSpec   *pspec)
 {
 	GUsbContext *context = G_USB_CONTEXT (object);
 	GUsbContextPrivate *priv = context->priv;
 
 	switch (prop_id) {
-	case PROP_LIBUSB_CONTEXT:
-		priv->ctx = g_value_get_pointer (value);
-		break;
 	case PROP_DEBUG_LEVEL:
 		priv->debug_level = g_value_get_int (value);
 		libusb_set_debug (priv->ctx, priv->debug_level);
@@ -123,36 +151,31 @@ g_usb_context_set_property (GObject		*object,
 	}
 }
 
-/**
- * usb_context_class_init:
- **/
 static void
 g_usb_context_class_init (GUsbContextClass *klass)
 {
-	GParamSpec *pspec;
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	object_class->finalize		= g_usb_context_finalize;
-	object_class->get_property	= g_usb_context_get_property;
-	object_class->set_property	= g_usb_context_set_property;
+	object_class->dispose = g_usb_context_dispose;
+	object_class->get_property = g_usb_context_get_property;
+	object_class->set_property = g_usb_context_set_property;
 
 	/**
 	 * GUsbContext:libusb_context:
 	 */
-	pspec = g_param_spec_pointer ("libusb_context", NULL, NULL,
-				      G_PARAM_CONSTRUCT_ONLY|
-				      G_PARAM_READWRITE);
-	g_object_class_install_property (object_class, PROP_LIBUSB_CONTEXT,
-					 pspec);
+	pspecs[PROP_LIBUSB_CONTEXT] =
+		g_param_spec_pointer ("libusb_context", NULL, NULL,
+		                      G_PARAM_READABLE);
 
 	/**
 	 * GUsbContext:debug_level:
 	 */
-	pspec = g_param_spec_int ("debug_level", NULL, NULL,
-				  0, 3, 0,
-				  G_PARAM_READWRITE);
-	g_object_class_install_property (object_class, PROP_DEBUG_LEVEL,
-					 pspec);
+	pspecs[PROP_DEBUG_LEVEL] =
+		g_param_spec_int ("debug_level", NULL, NULL,
+		                  0, 3, 0,
+		                  G_PARAM_READWRITE);
+
+	g_object_class_install_properties (object_class, N_PROPERTIES, pspecs);
 
 	/**
 	 * GUsbContext::device-added:
@@ -189,8 +212,6 @@ g_usb_context_class_init (GUsbContextClass *klass)
 			G_TYPE_NONE,
 			1,
 			G_USB_TYPE_DEVICE);
-
-	g_type_class_add_private (klass, sizeof (GUsbContextPrivate));
 }
 
 typedef struct {
@@ -199,9 +220,6 @@ typedef struct {
 	guint		 signal_id;
 } GUsbContextIdleHelper;
 
-/**
- * g_usb_context_idle_helper_free:
- **/
 static void
 g_usb_context_idle_helper_free (GUsbContextIdleHelper *helper)
 {
@@ -210,9 +228,6 @@ g_usb_context_idle_helper_free (GUsbContextIdleHelper *helper)
 	g_free (helper);
 }
 
-/**
- * g_usb_context_idle_signal_cb:
- **/
 static gboolean
 g_usb_context_idle_signal_cb (gpointer user_data)
 {
@@ -222,11 +237,9 @@ g_usb_context_idle_signal_cb (gpointer user_data)
 	return FALSE;
 }
 
-/**
- * g_usb_context_emit_device_add:
- **/
 static void
-g_usb_context_emit_device_add (GUsbContext *context, GUsbDevice *device)
+g_usb_context_emit_device_add (GUsbContext *context,
+                               GUsbDevice  *device)
 {
 	GUsbContextIdleHelper *helper;
 	helper = g_new0 (GUsbContextIdleHelper, 1);
@@ -240,11 +253,9 @@ g_usb_context_emit_device_add (GUsbContext *context, GUsbDevice *device)
 	g_idle_add (g_usb_context_idle_signal_cb, helper);
 }
 
-/**
- * g_usb_context_emit_device_remove:
- **/
 static void
-g_usb_context_emit_device_remove (GUsbContext *context, GUsbDevice *device)
+g_usb_context_emit_device_remove (GUsbContext *context,
+                                  GUsbDevice  *device)
 {
 	GUsbContextIdleHelper *helper;
 	helper = g_new0 (GUsbContextIdleHelper, 1);
@@ -258,21 +269,20 @@ g_usb_context_emit_device_remove (GUsbContext *context, GUsbDevice *device)
 	g_idle_add (g_usb_context_idle_signal_cb, helper);
 }
 
-/**
- * g_usb_context_add_device:
- **/
 static void
-g_usb_context_add_device (GUsbContext *context, struct libusb_device *dev)
+g_usb_context_add_device (GUsbContext          *context,
+                          struct libusb_device *dev)
 {
 	GUsbDevice *device = NULL;
 	GUsbContextPrivate *priv = context->priv;
-	gchar *platform_id = NULL;
 	guint8 bus;
 	guint8 address;
+	GError *error = NULL;
 
 	/* does any existing device exist */
 	bus = libusb_get_bus_number (dev);
 	address = libusb_get_device_address (dev);
+
 	if (priv->done_enumerate)
 		device = g_usb_context_find_by_bus_address (context, bus, address, NULL);
 	if (device != NULL) {
@@ -281,10 +291,11 @@ g_usb_context_add_device (GUsbContext *context, struct libusb_device *dev)
 	}
 
 	/* add the device */
-	platform_id = g_strdup_printf ("usb[%02x:%02x]", bus, address);
-	device = _g_usb_device_new (context, dev, platform_id);
-	if (g_usb_device_get_device_class (device) == 0x09) {
-		g_debug ("%02x:%02x is a hub, ignoring", bus, address);
+	device = _g_usb_device_new (context, dev, &error);
+	if (device == NULL) {
+		g_debug ("There was a problem creating the device: %s",
+		         error->message);
+		g_error_free (error);
 		goto out;
 	}
 	g_ptr_array_add (priv->devices, g_object_ref (device));
@@ -292,14 +303,11 @@ g_usb_context_add_device (GUsbContext *context, struct libusb_device *dev)
 out:
 	if (device != NULL)
 		g_object_unref (device);
-	g_free (platform_id);
 }
 
-/**
- * g_usb_context_remove_device:
- **/
 static void
-g_usb_context_remove_device (GUsbContext *context, struct libusb_device *dev)
+g_usb_context_remove_device (GUsbContext          *context,
+                             struct libusb_device *dev)
 {
 	GUsbDevice *device = NULL;
 	GUsbContextPrivate *priv = context->priv;
@@ -319,14 +327,11 @@ g_usb_context_remove_device (GUsbContext *context, struct libusb_device *dev)
 	g_object_unref (device);
 }
 
-/**
- * g_usb_context_hotplug_cb:
- **/
 static int
 g_usb_context_hotplug_cb (struct libusb_context *ctx,
-			  struct libusb_device *dev,
-			  libusb_hotplug_event event,
-			  void *user_data)
+                          struct libusb_device  *dev,
+                          libusb_hotplug_event   event,
+                          void                  *user_data)
 {
 	GUsbContext *context = G_USB_CONTEXT (user_data);
 
@@ -343,9 +348,6 @@ g_usb_context_hotplug_cb (struct libusb_context *ctx,
 	return 0;
 }
 
-/**
- * g_usb_context_rescan:
- **/
 static void
 g_usb_context_rescan (GUsbContext *context)
 {
@@ -357,11 +359,6 @@ g_usb_context_rescan (GUsbContext *context)
 	guint i;
 	libusb_device **dev_list = NULL;
 
-	/* add any devices not yet added (duplicates will be filtered */
-	libusb_get_device_list (priv->ctx, &dev_list);
-	for (i = 0; dev_list && dev_list[i]; i++)
-		g_usb_context_add_device (context, dev_list[i]);
-
 	/* copy to a context so we can remove from the array */
 	for (i = 0; i < priv->devices->len; i++) {
 		device = g_ptr_array_index (priv->devices, i);
@@ -385,13 +382,15 @@ g_usb_context_rescan (GUsbContext *context)
 		}
 	}
 
+	/* add any devices not yet added (duplicates will be filtered */
+	libusb_get_device_list (priv->ctx, &dev_list);
+	for (i = 0; dev_list && dev_list[i]; i++)
+		g_usb_context_add_device (context, dev_list[i]);
+
 	g_list_free (existing_devices);
 	libusb_free_device_list (dev_list, 1);
 }
 
-/**
- * g_usb_context_rescan_cb:
- **/
 static gboolean
 g_usb_context_rescan_cb (gpointer user_data)
 {
@@ -430,91 +429,81 @@ g_usb_context_enumerate (GUsbContext *context)
 	priv->done_enumerate = TRUE;
 }
 
-/**
- * g_usb_context_event_thread_cb:
- **/
 static gpointer
 g_usb_context_event_thread_cb (gpointer data)
 {
 	GUsbContext *context = G_USB_CONTEXT (data);
 	GUsbContextPrivate *priv = context->priv;
-	while (priv->thread_event_run)
+
+	while (g_atomic_int_get (&priv->thread_event_run) > 0)
 		libusb_handle_events (priv->ctx);
+
 	return NULL;
 }
 
-/**
- * g_usb_context_init:
- **/
 static void
 g_usb_context_init (GUsbContext *context)
 {
 	GUsbContextPrivate *priv;
-	gint rc;
 
-	priv = context->priv = G_USB_CONTEXT_GET_PRIVATE (context);
+	priv = context->priv = g_usb_context_get_instance_private (context);
 	priv->devices = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
-	priv->thread_event_run = TRUE;
+	priv->dict_usb_ids = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+}
+
+static gboolean
+g_usb_context_initable_init (GInitable     *initable,
+                             GCancellable  *cancellable,
+                             GError       **error)
+{
+	GUsbContext *context = G_USB_CONTEXT (initable);
+	GUsbContextPrivate *priv;
+	gint rc;
+	libusb_context *ctx;
+
+	priv = context->priv;
+
+	rc = libusb_init (&ctx);
+	if (rc < 0) {
+		g_set_error (error,
+		             G_USB_CONTEXT_ERROR,
+		             G_USB_CONTEXT_ERROR_INTERNAL,
+		             "failed to init libusb: %s [%i]",
+		             g_usb_strerror (rc), rc);
+		return FALSE;
+	}
+
+	priv->ctx = ctx;
+	priv->thread_event_run = 1;
 	priv->thread_event = g_thread_new ("GUsbEventThread",
-					   g_usb_context_event_thread_cb,
-					   context);
+	                                   g_usb_context_event_thread_cb,
+	                                   context);
 
 	/* watch for add/remove */
 	if (libusb_has_capability (LIBUSB_CAP_HAS_HOTPLUG)) {
 		rc = libusb_hotplug_register_callback (priv->ctx,
-						       LIBUSB_HOTPLUG_EVENT_DEVICE_ARRIVED |
-						       LIBUSB_HOTPLUG_EVENT_DEVICE_LEFT,
-						       0,
-						       LIBUSB_HOTPLUG_MATCH_ANY,
-						       LIBUSB_HOTPLUG_MATCH_ANY,
-						       LIBUSB_HOTPLUG_MATCH_ANY,
-						       g_usb_context_hotplug_cb,
-						       context,
-						       &context->priv->hotplug_id);
+		                                       LIBUSB_HOTPLUG_EVENT_DEVICE_ARRIVED |
+		                                       LIBUSB_HOTPLUG_EVENT_DEVICE_LEFT,
+		                                       0,
+		                                       LIBUSB_HOTPLUG_MATCH_ANY,
+		                                       LIBUSB_HOTPLUG_MATCH_ANY,
+		                                       LIBUSB_HOTPLUG_MATCH_ANY,
+		                                       g_usb_context_hotplug_cb,
+		                                       context,
+		                                       &context->priv->hotplug_id);
 		if (rc != LIBUSB_SUCCESS) {
 			g_warning ("Error creating a hotplug callback: %s",
-				   g_usb_strerror (rc));
+			           g_usb_strerror (rc));
 		}
 	}
+
+	return TRUE;
 }
 
-/**
- * g_usb_context_finalize:
- **/
 static void
-g_usb_context_finalize (GObject *object)
+g_usb_context_initable_iface_init (GInitableIface *iface)
 {
-	GUsbContext *context = G_USB_CONTEXT (object);
-	GUsbContextPrivate *priv = context->priv;
-
-	/* this is safe to call even when priv->hotplug_id is unset */
-	priv->thread_event_run = FALSE;
-	libusb_hotplug_deregister_callback (priv->ctx, priv->hotplug_id);
-	g_thread_join (priv->thread_event);
-
-	if (priv->hotplug_poll_id > 0)
-		g_source_remove (priv->hotplug_poll_id);
-	g_ptr_array_unref (priv->devices);
-
-	libusb_exit (priv->ctx);
-
-	G_OBJECT_CLASS (g_usb_context_parent_class)->finalize (object);
-}
-
-/**
- * g_usb_context_error_quark:
- *
- * Return value: Our personal error quark.
- *
- * Since: 0.1.0
- **/
-GQuark
-g_usb_context_error_quark (void)
-{
-	static GQuark quark = 0;
-	if (!quark)
-		quark = g_quark_from_static_string ("g_usb_context_error");
-	return quark;
+	iface->init = g_usb_context_initable_init;
 }
 
 /**
@@ -545,7 +534,8 @@ _g_usb_context_get_context (GUsbContext *context)
  * Since: 0.1.0
  **/
 GUsbSource *
-g_usb_context_get_source (GUsbContext *context, GMainContext *main_ctx)
+g_usb_context_get_source (GUsbContext  *context,
+                          GMainContext *main_ctx)
 {
 	return NULL;
 }
@@ -563,20 +553,31 @@ g_usb_context_get_source (GUsbContext *context, GMainContext *main_ctx)
  * Since: 0.1.0
  **/
 void
-g_usb_context_set_debug (GUsbContext *context, GLogLevelFlags flags)
+g_usb_context_set_debug (GUsbContext    *context,
+                         GLogLevelFlags  flags)
 {
-	GUsbContextPrivate *priv = context->priv;
+	GUsbContextPrivate *priv;
+	int debug_level;
+
+	g_return_if_fail (G_USB_IS_CONTEXT (context));
+
+	priv = context->priv;
 
 	if (flags & (G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_INFO))
-		priv->debug_level = 3;
+		debug_level = 3;
 	else if (flags & (G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_WARNING))
-		priv->debug_level = 2;
+		debug_level = 2;
 	else if (flags & (G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR))
-		priv->debug_level = 1;
+		debug_level = 1;
 	else
-		priv->debug_level = 0;
+		debug_level = 0;
 
-	libusb_set_debug (priv->ctx, priv->debug_level);
+	if (debug_level != priv->debug_level) {
+		priv->debug_level = debug_level;
+		libusb_set_debug (priv->ctx, debug_level);
+
+		g_object_notify_by_pspec (G_OBJECT (context), pspecs[PROP_DEBUG_LEVEL]);
+	}
 }
 
 /**
@@ -593,30 +594,84 @@ g_usb_context_set_debug (GUsbContext *context, GLogLevelFlags flags)
  * Since: 0.2.2
  **/
 GUsbDevice *
-g_usb_context_find_by_bus_address (GUsbContext	*context,
-				   guint8	 bus,
-				   guint8	 address,
-				   GError	**error)
+g_usb_context_find_by_bus_address (GUsbContext  *context,
+                                   guint8        bus,
+                                   guint8        address,
+                                   GError      **error)
 {
-	GUsbContextPrivate *priv = context->priv;
+	GUsbContextPrivate *priv;
 	GUsbDevice *device = NULL;
 	guint i;
 
+	g_return_val_if_fail (G_USB_IS_CONTEXT (context), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	priv = context->priv;
+
 	g_usb_context_enumerate (context);
 	for (i = 0; i < priv->devices->len; i++) {
 		GUsbDevice *curr = g_ptr_array_index (priv->devices, i);
 		if (g_usb_device_get_bus (curr) == bus &&
 		    g_usb_device_get_address (curr) == address) {
 			device = g_object_ref (curr);
-			goto out;
+			break;
 		}
 	}
-	g_set_error (error,
-		     G_USB_DEVICE_ERROR,
-		     G_USB_DEVICE_ERROR_NO_DEVICE,
-		     "Failed to find device %04x:%04x",
-		     bus, address);
-out:
+
+	if (device == NULL) {
+		g_set_error (error,
+		             G_USB_DEVICE_ERROR,
+		             G_USB_DEVICE_ERROR_NO_DEVICE,
+		             "Failed to find device %04x:%04x",
+		             bus, address);
+	}
+
+	return device;
+}
+
+/**
+ * g_usb_context_find_by_platform_id:
+ * @context: a #GUsbContext
+ * @platform_id: a platform id, e.g. "usb:00:03:03:02"
+ * @error: A #GError or %NULL
+ *
+ * Finds a device based on its platform id value.
+ *
+ * Return value: (transfer full): a new #GUsbDevice, or %NULL if not found.
+ *
+ * Since: 0.2.4
+ **/
+GUsbDevice *
+g_usb_context_find_by_platform_id (GUsbContext *context,
+                                   const gchar *platform_id,
+                                   GError      **error)
+{
+	GUsbContextPrivate *priv;
+	GUsbDevice *device = NULL;
+	guint i;
+
+	g_return_val_if_fail (G_USB_IS_CONTEXT (context), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	priv = context->priv;
+
+	g_usb_context_enumerate (context);
+	for (i = 0; i < priv->devices->len; i++) {
+		GUsbDevice *curr = g_ptr_array_index (priv->devices, i);
+		if (g_strcmp0 (g_usb_device_get_platform_id (curr), platform_id) == 0) {
+			device = g_object_ref (curr);
+			break;
+		}
+	}
+
+	if (device == NULL) {
+		g_set_error (error,
+		             G_USB_DEVICE_ERROR,
+		             G_USB_DEVICE_ERROR_NO_DEVICE,
+		             "Failed to find device %s",
+		             platform_id);
+	}
+
 	return device;
 }
 
@@ -634,32 +689,184 @@ out:
  * Since: 0.2.2
  **/
 GUsbDevice *
-g_usb_context_find_by_vid_pid (GUsbContext	*context,
-			       guint16		 vid,
-			       guint16		 pid,
-			       GError		**error)
+g_usb_context_find_by_vid_pid (GUsbContext  *context,
+                               guint16       vid,
+                               guint16       pid,
+                               GError      **error)
 {
-	GUsbContextPrivate *priv = context->priv;
+	GUsbContextPrivate *priv;
 	GUsbDevice *device = NULL;
 	guint i;
 
+	g_return_val_if_fail (G_USB_IS_CONTEXT (context), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	priv = context->priv;
+
 	g_usb_context_enumerate (context);
 	for (i = 0; i < priv->devices->len; i++) {
 		GUsbDevice *curr = g_ptr_array_index (priv->devices, i);
 		if (g_usb_device_get_vid (curr) == vid &&
 		    g_usb_device_get_pid (curr) == pid) {
 			device = g_object_ref (curr);
-			goto out;
+			break;
 		}
 	}
-	g_set_error (error,
-		     G_USB_DEVICE_ERROR,
-		     G_USB_DEVICE_ERROR_NO_DEVICE,
-		     "Failed to find device %04x:%04x",
-		     vid, pid);
-out:
+
+	if (device == NULL) {
+		g_set_error (error,
+		             G_USB_DEVICE_ERROR,
+		             G_USB_DEVICE_ERROR_NO_DEVICE,
+		             "Failed to find device %04x:%04x",
+		             vid, pid);
+	}
+
 	return device;
 }
+static gboolean
+g_usb_context_load_usb_ids (GUsbContext  *context,
+                            GError      **error)
+{
+	guint16 pid;
+	guint16 vid = 0x0000;
+	guint i;
+	gchar *data = NULL;
+	gchar **lines = NULL;
+
+	/* already loaded */
+	if (g_hash_table_size (context->priv->dict_usb_ids) > 0)
+		return TRUE;
+
+	/* parse */
+	if (!g_file_get_contents ("/usr/share/hwdata/usb.ids", &data, NULL, error))
+		return FALSE;
+
+	lines = g_strsplit (data, "\n", -1);
+	g_free (data);
+
+	for (i = 0; lines[i] != NULL; i++) {
+		if (lines[i][0] == '#')
+			continue;
+
+		if (lines[i][0] == '\0')
+			continue;
+
+		if (lines[i][0] != '\t') {
+			lines[i][4] = '\0';
+
+			vid = g_ascii_strtoull (lines[i], NULL, 16);
+			if (vid == 0)
+				break;
+
+			g_hash_table_insert (context->priv->dict_usb_ids,
+			                     g_strdup (lines[i]),
+			                     g_strdup (lines[i] + 6));
+		} else {
+			if (vid == 0x0000)
+				break;
+
+			lines[i][5] = '\0';
+			pid = g_ascii_strtoull (lines[i] + 1, NULL, 16);
+			g_hash_table_insert (context->priv->dict_usb_ids,
+			                     g_strdup_printf ("%04x:%04x", vid, pid),
+			                     g_strdup (lines[i] + 7));
+		}
+	}
+
+	g_strfreev (lines);
+
+	return TRUE;
+}
+
+/**
+ * _g_usb_context_lookup_vendor:
+ * @context: a #GUsbContext
+ * @vid: a USB vendor ID
+ * @error: a #GError, or %NULL
+ *
+ * Returns the vendor name using usb.ids.
+ *
+ * Return value: the description, or %NULL
+ *
+ * Since: 0.1.5
+ **/
+const gchar *
+_g_usb_context_lookup_vendor (GUsbContext  *context,
+                              guint16       vid,
+                              GError      **error)
+{
+	const gchar *tmp;
+	gchar *key = NULL;
+
+	g_return_val_if_fail (G_USB_IS_CONTEXT (context), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	/* load */
+	if (!g_usb_context_load_usb_ids (context, error))
+		return NULL;
+
+	/* find */
+	key = g_strdup_printf ("%04x", vid);
+	tmp = g_hash_table_lookup (context->priv->dict_usb_ids, key);
+	if (tmp == NULL) {
+		g_set_error (error,
+		             G_USB_CONTEXT_ERROR,
+		             G_USB_CONTEXT_ERROR_INTERNAL,
+		             "failed to find vid %s", key);
+		g_free (key);
+		return NULL;
+	}
+
+	g_free (key);
+
+	return tmp;
+}
+
+/**
+ * _g_usb_context_lookup_product:
+ * @context: a #GUsbContext
+ * @vid: a USB vendor ID
+ * @pid: a USB product ID
+ * @error: a #GError, or %NULL
+ *
+ * Returns the product name using usb.ids.
+ *
+ * Return value: the description, or %NULL
+ *
+ * Since: 0.1.5
+ **/
+const gchar *
+_g_usb_context_lookup_product (GUsbContext  *context,
+                               guint16       vid,
+                               guint16       pid,
+                               GError      **error)
+{
+	const gchar *tmp;
+	gchar *key = NULL;
+
+	g_return_val_if_fail (G_USB_IS_CONTEXT (context), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	/* load */
+	if (!g_usb_context_load_usb_ids (context, error))
+		return NULL;
+
+	/* find */
+	key = g_strdup_printf ("%04x:%04x", vid, pid);
+	tmp = g_hash_table_lookup (context->priv->dict_usb_ids, key);
+	if (tmp == NULL) {
+		g_set_error (error,
+		             G_USB_CONTEXT_ERROR,
+		             G_USB_CONTEXT_ERROR_INTERNAL,
+		             "failed to find vid %s", key);
+		g_free (key);
+		return NULL;
+	}
+
+	g_free (key);
+
+	return tmp;
+}
 
 /**
  * g_usb_context_get_devices:
@@ -673,7 +880,9 @@ GPtrArray *
 g_usb_context_get_devices (GUsbContext *context)
 {
 	g_return_val_if_fail (G_USB_IS_CONTEXT (context), NULL);
+
 	g_usb_context_enumerate (context);
+
 	return g_ptr_array_ref (context->priv->devices);
 }
 
@@ -690,19 +899,5 @@ g_usb_context_get_devices (GUsbContext *context)
 GUsbContext *
 g_usb_context_new (GError **error)
 {
-	gint rc;
-	GObject *obj;
-	libusb_context *ctx;
-
-	rc = libusb_init (&ctx);
-	if (rc < 0) {
-		g_set_error (error,
-			     G_USB_CONTEXT_ERROR,
-			     G_USB_CONTEXT_ERROR_INTERNAL,
-			     "failed to init libusb: %s [%i]",
-			     g_usb_strerror (rc), rc);
-		return NULL;
-	}
-	obj = g_object_new (G_USB_TYPE_CONTEXT, "libusb_context", ctx, NULL);
-	return G_USB_CONTEXT (obj);
+	return g_initable_new (G_USB_TYPE_CONTEXT, NULL, error, NULL);
 }
