@@ -38,8 +38,6 @@
 #include "gusb-device-list.h"
 #include "gusb-device-private.h"
 
-#define G_USB_DEVICE_LIST_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), G_USB_TYPE_DEVICE_LIST, GUsbDeviceListPrivate))
-
 enum {
 	PROP_0,
 	PROP_CONTEXT
@@ -53,27 +51,18 @@ enum
 };
 
 struct _GUsbDeviceListPrivate {
-	GUsbContext			*context;
+	GUsbContext *context;
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-G_DEFINE_TYPE (GUsbDeviceList, g_usb_device_list, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (GUsbDeviceList, g_usb_device_list, G_TYPE_OBJECT);
 
 static void
-g_usb_device_list_finalize (GObject *object)
-{
-	G_OBJECT_CLASS (g_usb_device_list_parent_class)->finalize (object);
-}
-
-/**
- * g_usb_device_list_get_property:
- **/
-static void
-g_usb_device_list_get_property (GObject		*object,
-				guint		 prop_id,
-				GValue		*value,
-				GParamSpec	*pspec)
+g_usb_device_list_get_property (GObject    *object,
+                                guint       prop_id,
+                                GValue     *value,
+                                GParamSpec *pspec)
 {
 	GUsbDeviceList *list = G_USB_DEVICE_LIST (object);
 	GUsbDeviceListPrivate *priv = list->priv;
@@ -88,32 +77,27 @@ g_usb_device_list_get_property (GObject		*object,
 	}
 }
 
-/**
- * g_usb_device_added_cb:
- **/
 static void
-g_usb_device_added_cb (GUsbContext *context, GUsbDevice *device, GUsbDeviceList *list)
+g_usb_device_added_cb (GUsbContext    *context,
+                       GUsbDevice     *device,
+                       GUsbDeviceList *list)
 {
 	g_signal_emit (list, signals[DEVICE_ADDED_SIGNAL], 0, device);
 }
 
-/**
- * g_usb_device_removed_cb:
- **/
 static void
-g_usb_device_removed_cb (GUsbContext *context, GUsbDevice *device, GUsbDeviceList *list)
+g_usb_device_removed_cb (GUsbContext    *context,
+                         GUsbDevice     *device,
+                         GUsbDeviceList *list)
 {
 	g_signal_emit (list, signals[DEVICE_REMOVED_SIGNAL], 0, device);
 }
 
-/**
- * usb_device_list_set_property:
- **/
 static void
-g_usb_device_list_set_property (GObject		*object,
-				guint		 prop_id,
-				const GValue	*value,
-				GParamSpec	*pspec)
+g_usb_device_list_set_property (GObject      *object,
+                                guint         prop_id,
+                                const GValue *value,
+                                GParamSpec   *pspec)
 {
 	GUsbDeviceList *list = G_USB_DEVICE_LIST (object);
 	GUsbDeviceListPrivate *priv = list->priv;
@@ -132,16 +116,12 @@ g_usb_device_list_set_property (GObject		*object,
 	}
 }
 
-/**
- * g_usb_device_list_class_init:
- **/
 static void
 g_usb_device_list_class_init (GUsbDeviceListClass *klass)
 {
 	GParamSpec *pspec;
-	GObjectClass *object_class = (GObjectClass *) klass;
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	object_class->finalize		= g_usb_device_list_finalize;
 	object_class->get_property	= g_usb_device_list_get_property;
 	object_class->set_property	= g_usb_device_list_set_property;
 
@@ -189,17 +169,12 @@ g_usb_device_list_class_init (GUsbDeviceListClass *klass)
 			G_TYPE_NONE,
 			1,
 			G_USB_TYPE_DEVICE);
-
-	g_type_class_add_private (klass, sizeof (GUsbDeviceListPrivate));
 }
 
-/**
- * g_usb_device_list_class_init:
- **/
 static void
 g_usb_device_list_init (GUsbDeviceList *list)
 {
-	list->priv = G_USB_DEVICE_LIST_GET_PRIVATE (list);
+	list->priv = g_usb_device_list_get_instance_private (list);
 }
 
 /**
@@ -213,6 +188,8 @@ g_usb_device_list_init (GUsbDeviceList *list)
 GPtrArray *
 g_usb_device_list_get_devices (GUsbDeviceList *list)
 {
+	g_return_val_if_fail (G_USB_IS_DEVICE_LIST (list), NULL);
+
 	return g_usb_context_get_devices (list->priv->context);
 }
 
@@ -244,12 +221,16 @@ g_usb_device_list_coldplug (GUsbDeviceList *list)
  * Since: 0.1.0
  **/
 GUsbDevice *
-g_usb_device_list_find_by_bus_address (GUsbDeviceList	*list,
-				       guint8		 bus,
-				       guint8		 address,
-				       GError		**error)
+g_usb_device_list_find_by_bus_address (GUsbDeviceList  *list,
+                                       guint8           bus,
+                                       guint8           address,
+                                       GError         **error)
 {
-	return g_usb_context_find_by_bus_address (list->priv->context, bus, address, error);
+	g_return_val_if_fail (G_USB_IS_DEVICE_LIST (list), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	return g_usb_context_find_by_bus_address (list->priv->context,
+	                                          bus, address, error);
 }
 
 /**
@@ -266,12 +247,16 @@ g_usb_device_list_find_by_bus_address (GUsbDeviceList	*list,
  * Since: 0.1.0
  **/
 GUsbDevice *
-g_usb_device_list_find_by_vid_pid (GUsbDeviceList	*list,
-				   guint16		 vid,
-				   guint16		 pid,
-				   GError		**error)
+g_usb_device_list_find_by_vid_pid (GUsbDeviceList  *list,
+                                   guint16          vid,
+                                   guint16          pid,
+                                   GError         **error)
 {
-	return g_usb_context_find_by_vid_pid (list->priv->context, vid, pid, error);
+	g_return_val_if_fail (G_USB_IS_DEVICE_LIST (list), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	return g_usb_context_find_by_vid_pid (list->priv->context,
+	                                      vid, pid, error);
 }
 
 /**
@@ -290,7 +275,5 @@ g_usb_device_list_find_by_vid_pid (GUsbDeviceList	*list,
 GUsbDeviceList *
 g_usb_device_list_new (GUsbContext *context)
 {
-	GObject *obj;
-	obj = g_object_new (G_USB_TYPE_DEVICE_LIST, "context", context, NULL);
-	return G_USB_DEVICE_LIST (obj);
+	return g_object_new (G_USB_TYPE_DEVICE_LIST, "context", context, NULL);
 }
