@@ -121,13 +121,20 @@ GType			 g_usb_device_get_type		(void);
 GQuark			 g_usb_device_error_quark	(void);
 
 const gchar		*g_usb_device_get_platform_id	(GUsbDevice	*device);
+GUsbDevice		*g_usb_device_get_parent	(GUsbDevice	*device);
+GPtrArray		*g_usb_device_get_children	(GUsbDevice	*device);
 
 guint8			 g_usb_device_get_bus		(GUsbDevice	*device);
 guint8			 g_usb_device_get_address	(GUsbDevice	*device);
+guint8			 g_usb_device_get_port_number	(GUsbDevice	*device);
 
 guint16			 g_usb_device_get_vid		(GUsbDevice	*device);
 guint16			 g_usb_device_get_pid		(GUsbDevice	*device);
+const gchar		*g_usb_device_get_vid_as_str	(GUsbDevice	*device);
+const gchar		*g_usb_device_get_pid_as_str	(GUsbDevice	*device);
 guint8			 g_usb_device_get_device_class	(GUsbDevice	*device);
+guint8			 g_usb_device_get_device_subclass       (GUsbDevice *device);
+guint8                   g_usb_device_get_device_protocol       (GUsbDevice *device);
 
 guint8			 g_usb_device_get_manufacturer_index	(GUsbDevice *device);
 guint8			 g_usb_device_get_product_index		(GUsbDevice *device);
