@@ -24,6 +24,7 @@
 #include <glib-object.h>
 
 #include "gusb-context.h"
+#include "gusb-context-private.h"
 #include "gusb-device.h"
 
 static void
@@ -51,6 +52,26 @@ gusb_device_func (void)
 	g_ptr_array_unref (array);
 }
 
+static void
+gusb_context_lookup_func (void)
+{
+	GUsbContext *ctx = NULL;
+	GError *error = NULL;
+	const gchar *tmp;
+
+	ctx = g_usb_context_new (&error);
+	g_assert_no_error (error);
+	g_assert (ctx != NULL);
+
+	tmp = _g_usb_context_lookup_vendor (ctx, 0x04d8, &error);
+	g_assert_no_error (error);
+	g_assert_cmpstr (tmp, ==, "Microchip Technology, Inc.");
+	tmp = _g_usb_context_lookup_product (ctx, 0x04d8, 0xf8da, &error);
+	g_assert_no_error (error);
+	g_assert_cmpstr (tmp, ==, "Hughski Ltd. ColorHug");
+	g_object_unref (ctx);
+}
+
 static void
 gusb_context_func (void)
 {
@@ -268,9 +289,9 @@ typedef struct {
 
 
 static void
-g_usb_device_print_data (const gchar *title,
-			 const guchar *data,
-			 gsize length)
+g_usb_device_print_data (const gchar  *title,
+                         const guchar *data,
+                         gsize         length)
 {
 	guint i;
 
@@ -289,9 +310,9 @@ g_usb_device_print_data (const gchar *title,
 }
 
 static void
-g_usb_test_button_pressed_cb (GObject *source_object,
-			      GAsyncResult *res,
-			      gpointer user_data)
+g_usb_test_button_pressed_cb (GObject      *source_object,
+                              GAsyncResult *res,
+                              gpointer      user_data)
 {
 	gboolean ret;
 	GError *error = NULL;
@@ -444,7 +465,8 @@ out:
 }
 
 int
-main (int argc, char **argv)
+main (int    argc,
+      char **argv)
 {
 	g_test_init (&argc, &argv, NULL);
 
@@ -453,6 +475,7 @@ main (int argc, char **argv)
 
 	/* tests go here */
 	g_test_add_func ("/gusb/context", gusb_context_func);
+	g_test_add_func ("/gusb/context{lookup}", gusb_context_lookup_func);
 	g_test_add_func ("/gusb/device", gusb_device_func);
 	g_test_add_func ("/gusb/device[huey]", gusb_device_huey_func);
 	g_test_add_func ("/gusb/device[munki]", gusb_device_munki_func);
